@@ -24,7 +24,6 @@
  *  aptmethod.c		start, feed and take care of the apt methods
  *  downloadcache.c     keep track of what is downloaded to avoid duplicates
  *  signature.c		verify Release.gpg files, if requested
- *  readrelease.c	parse Release files, unless ignored
  *  remoterepository.c  cache remote index files and decide which to download
  *  upgradelist.c	decide which packages (and version) should be installed
  *
@@ -76,7 +75,6 @@
 #include "tracking.h"
 #include "termdecide.h"
 #include "filterlist.h"
-#include "readrelease.h"
 #include "log.h"
 #include "donefile.h"
 #include "freespace.h"
@@ -169,11 +167,14 @@ struct update_pattern {
 	component_t flat;
 	//e.g. "IgnoreRelease: Yes" for 1 (default is 0)
 	bool ignorerelease;
+	//e.g. "GetInRelease: No" for 0 (default is 1)
+	bool getinrelease;
 	/* the form in which index files are preferably downloaded */
 	struct encoding_preferences downloadlistsas;
 	/* if the specific field is there (to destinguish from an empty one) */
 	bool ignorehashes_set;
 	bool ignorerelease_set;
+	bool getinrelease_set;
 	bool architectures_set;
 	bool components_set;
 	bool udebcomponents_set;
@@ -351,6 +352,7 @@ CFurlSETPROC(update_pattern, fallback)
 CFallSETPROC(update_pattern, verifyrelease)
 CFlinelistSETPROC(update_pattern, config)
 CFtruthSETPROC(update_pattern, ignorerelease)
+CFtruthSETPROC(update_pattern, getinrelease)
 CFscriptSETPROC(update_pattern, listhook)
 CFallSETPROC(update_pattern, shellhook)
 CFfilterlistSETPROC(update_pattern, filterlist)
@@ -525,6 +527,7 @@ static const struct configfield updateconfigfields[] = {
 	CF("Components", update_pattern, components),
 	CF("Flat", update_pattern, flat),
 	CF("UDebComponents", update_pattern, udebcomponents),
+	CF("GetInRelease", update_pattern, getinrelease),
 	CF("IgnoreRelease", update_pattern, ignorerelease),
 	CF("IgnoreHashes", update_pattern, ignorehashes),
 	CF("VerifyRelease", update_pattern, verifyrelease),
@@ -760,7 +763,7 @@ static retvalue instance_pattern(struct update_pattern *pattern, const struct di
 
 	struct update_origin *update;
 	/*@dependant@*/struct update_pattern *declaration, *p, *listscomponents;
-	bool ignorehashes[cs_hashCOUNT], ignorerelease;
+	bool ignorehashes[cs_hashCOUNT], ignorerelease, getinrelease;
 	const char *verifyrelease;
 	retvalue r;
 
@@ -807,6 +810,13 @@ static retvalue instance_pattern(struct update_pattern *pattern, const struct di
 		ignorerelease = false;
 	else
 		ignorerelease = p->ignorerelease;
+	p = pattern;
+	while (p != NULL && !p->getinrelease_set)
+		p = p->pattern_from;
+	if (p == NULL)
+		getinrelease = true;
+	else
+		getinrelease = p->getinrelease;
 	/* find the first set values: */
 	p = pattern;
 	while (p != NULL && p->verifyrelease == NULL)
@@ -859,7 +869,7 @@ static retvalue instance_pattern(struct update_pattern *pattern, const struct di
 	}
 	r = remote_distribution_prepare(declaration->repository,
 			update->suite_from, ignorerelease,
-			verifyrelease, update->flat,
+			getinrelease, verifyrelease, update->flat,
 			ignorehashes, &update->from);
 	if (RET_WAS_ERROR(r)) {
 		updates_freeorigins(update);
@@ -1149,7 +1159,7 @@ static retvalue adddeleteruletotarget(struct update_target *updatetargets) {
 	return RET_OK;
 }
 
-static retvalue gettargets(struct update_origin *origins, struct distribution *distribution, struct update_target **ts) {
+static retvalue gettargets(struct update_origin *origins, struct distribution *distribution, const struct atomlist *components, const struct atomlist *architectures, const struct atomlist *types,  struct update_target **ts) {
 	struct target *target;
 	struct update_origin *origin;
 	struct update_target *updatetargets;
@@ -1159,6 +1169,8 @@ static retvalue gettargets(struct update_origin *origins, struct distribution *d
 
 	for (target = distribution->targets ; target != NULL ;
 	                                      target = target->next) {
+		if (!target_matches(target, components, architectures, types))
+			continue;
 		r = newupdatetarget(&updatetargets, target);
 		if (RET_WAS_ERROR(r)) {
 			updates_freetargets(updatetargets);
@@ -1231,7 +1243,7 @@ static inline retvalue findmissingupdate(const struct distribution *distribution
 	return result;
 }
 
-retvalue updates_calcindices(struct update_pattern *patterns, struct distribution *distributions, struct update_distribution **update_distributions) {
+retvalue updates_calcindices(struct update_pattern *patterns, struct distribution *distributions, const struct atomlist *components, const struct atomlist *architectures, const struct atomlist *types, struct update_distribution **update_distributions) {
 	struct distribution *distribution;
 	struct update_distribution *u_ds;
 	retvalue result, r;
@@ -1280,7 +1292,9 @@ retvalue updates_calcindices(struct update_pattern *patterns, struct distributio
 				break;
 			}
 
-			r = gettargets(u_d->origins, distribution, &u_d->targets);
+			r = gettargets(u_d->origins, distribution,
+					components, architectures, types,
+					&u_d->targets);
 			if (RET_WAS_ERROR(r)) {
 				result = r;
 				break;
@@ -1364,6 +1378,7 @@ static retvalue calllisthook(struct update_target *ut, struct update_index_conne
 		setenv("REPREPRO_BASE_DIR", global.basedir, true);
 		setenv("REPREPRO_OUT_DIR", global.outdir, true);
 		setenv("REPREPRO_CONF_DIR", global.confdir, true);
+		setenv("REPREPRO_CONFIG_DIR", global.confdir, true);
 		setenv("REPREPRO_DIST_DIR", global.distdir, true);
 		setenv("REPREPRO_LOG_DIR", global.logdir, true);
 		setenv("REPREPRO_FILTER_CODENAME",
@@ -1479,6 +1494,7 @@ static retvalue callshellhook(struct update_target *ut, struct update_index_conn
 		setenv("REPREPRO_BASE_DIR", global.basedir, true);
 		setenv("REPREPRO_OUT_DIR", global.outdir, true);
 		setenv("REPREPRO_CONF_DIR", global.confdir, true);
+		setenv("REPREPRO_CONFIG_DIR", global.confdir, true);
 		setenv("REPREPRO_DIST_DIR", global.distdir, true);
 		setenv("REPREPRO_LOG_DIR", global.logdir, true);
 		setenv("REPREPRO_FILTER_CODENAME",
